
package com.navigine.navigine;

import static com.navigine.navigine.Convert.bitmapToByteArray;

import android.content.Context;
import android.os.AsyncTask;
import android.os.Handler;
import android.os.Looper;
import android.util.Log;

import com.navigine.naviginesdk.DeviceInfo;
import com.navigine.naviginesdk.Location;
import com.navigine.naviginesdk.LocationPoint;
import com.navigine.naviginesdk.LocationView;
import com.navigine.naviginesdk.NavigationHelper;
import com.navigine.naviginesdk.NavigationThread;
import com.navigine.naviginesdk.NavigineNotification;
import com.navigine.naviginesdk.NavigineSDK;
import com.navigine.naviginesdk.NetworkUtils;
import com.navigine.naviginesdk.RoutePath;
import com.navigine.naviginesdk.SubLocation;

import java.util.Locale;

import io.flutter.plugin.common.MethodChannel;

class NavigineHelper {
    private final MethodChannel methodChannel;
    NavigationThread navigation;
    Context context;


    NavigineHelper(MethodChannel methodChannel) {
        this.methodChannel = methodChannel;
    }

    boolean initializeSDK(Context context, String userHash, String serverUrl) {
        this.context = context;
        return NavigineSDK.initialize(context, userHash, serverUrl);
    }


    void attachNotificationListener() {
        NavigationHelper.setNotificationListener(new NavigineNotification.Listener() {
            @Override
            public void onNotificationReceived(Context context, NavigineNotification navigineNotification) {
                if (methodChannel != null) {
                    Object notificationObject = Convert.notificationToObject(navigineNotification);
                    methodChannel.invokeMethod("marketing#onNotification", notificationObject);
                }
            }
        });
    }

    void removeNotificationListener() {
        NavigationHelper.setNotificationListener(null);
    }

    void loadLocation(final int locationId) {
        NavigineSDK.setParameter(context, "actions_updates_enabled", true);
        NavigineSDK.setParameter(context, "navigine_service_enabled", true);
        NavigineSDK.setParameter(context, "location_update_timeout", 10);
        NavigineSDK.loadLocationInBackground(locationId, new Location.LoadListener() {
            @Override
            public void onUpdate(int state) {
                super.onUpdate(state);
                methodChannel.invokeMethod("location#onUpdate", state);
            }

            @Override
            public void onFailed(int error) {
                super.onFailed(error);
                methodChannel.invokeMethod("location#onFailed", error);
            }

            @Override
            public void onFinished() {
                super.onFinished();
                methodChannel.invokeMethod("location#onFinished", null);
                navigation = NavigineSDK.getNavigation();
            }
        });
    }

    void startNavigation(int mode) {
        if (navigation == null) {
            navigation = NavigineSDK.getNavigation();
        }
        navigation.setMode(mode);
        navigation.setDeviceListener(new DeviceInfo.Listener() {
            @Override
            public void onUpdate(DeviceInfo info) {
                super.onUpdate(info);
                if (info.isValid()) {
                    methodChannel.invokeMethod("deviceInfo#onUpdate", Convert.deviceInfoToObject(navigation.getDeviceInfo()));
                }
            }
        });
    }

    void addTarget(LocationPoint point) {
        navigation.addTarget(point);
    }

    void cancelTarget() {
        navigation.cancelTarget();
    }

    RoutePath makeRoute(LocationPoint point1, LocationPoint point2) {
        return navigation.makeRoute(point1, point2);
    }

    Object getDeviceInfo() {
        return Convert.deviceInfoToObject(navigation.getDeviceInfo());
    }

    Object getLocation() {
        return Convert.locationToObject(navigation.getLocation());
    }

    void getSublocationImage(final int subLocationId, final OnImageReady onComplete) {
        new Thread(new Runnable() {
            @Override
            public void run() {
                Location location = navigation.getLocation();
                SubLocation subLocation = location.getSubLocation(subLocationId);
                onComplete.onReady(subLocation.getImage().getData());
            }
        }).start();
    }

    void stopNavigation() {
        navigation.interrupt();
        navigation = null;
    }
}


interface OnImageReady{
   void onReady(byte[] bytes);
}