package com.navigine.navigine;

import android.content.Context;
import android.util.Log;

import androidx.annotation.NonNull;

import com.navigine.naviginesdk.LocationPoint;
import com.navigine.naviginesdk.RoutePath;

import java.util.Map;

import io.flutter.embedding.engine.plugins.FlutterPlugin;
import io.flutter.embedding.engine.plugins.activity.ActivityAware;
import io.flutter.embedding.engine.plugins.activity.ActivityPluginBinding;
import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel;
import io.flutter.plugin.common.MethodChannel.MethodCallHandler;
import io.flutter.plugin.common.MethodChannel.Result;
import io.flutter.plugin.common.PluginRegistry.Registrar;

public class NaviginePlugin implements FlutterPlugin, ActivityAware, MethodCallHandler {
    NavigineHelper helper;
    Context context;
    MethodChannel channel;

    @Override
    public void onAttachedToEngine(@NonNull FlutterPluginBinding binding) {
        channel = new MethodChannel(binding.getBinaryMessenger(), "navigine_sdk");
        helper = new NavigineHelper(channel);
    }

    @Override
    public void onMethodCall(MethodCall call, final Result result) {
        String methodName = call.method;
        Log.d(" Navigine Flutter", System.currentTimeMillis() + call.method);
        switch (methodName) {
            case "initialize":
                result.success(helper.initializeSDK(context, (String) call.argument("userHash"),
                        (String) call.argument("serverUrl")));
                break;
            case "loadLocation":
                result.success(null);
                helper.loadLocation((int) call.argument("locationId"));
                break;
            case "startNavigation":
                helper.startNavigation((int) call.argument("mode"));
                result.success(null);
                break;
            case "stopNavigation":
                helper.stopNavigation();
                result.success(null);
                break;
            case "getLocation":
                result.success(helper.getLocation());
                break;
            case "getSublocationImage":
                helper.getSublocationImage((int) call.argument("subLocationId"), new OnImageReady(){
                    @Override
                    public void onReady(byte[] bytes) {
                        result.success(bytes);
                    }
                });
                break;
            case "getDeviceInfo":
                result.success(helper.getDeviceInfo());
                break;
            case "addTarget":
                helper.addTarget(Convert.objectToLocationPoint((Map<String, Object>) call.argument("locationPoint")));
                result.success(null);
                break;
            case "cancelTarget":
                helper.cancelTarget();
                result.success(null);
                break;
            case "attachNotificationListener":
                helper.attachNotificationListener();
                result.success(null);
                break;
            case "removeNotificationListener":
                helper.removeNotificationListener();
                result.success(null);
                break;
            case "makeRoute":
                LocationPoint point1 = Convert
                        .objectToLocationPoint((Map<String, Object>) call.argument("locationPoint1"));
                LocationPoint point2 = Convert
                        .objectToLocationPoint((Map<String, Object>) call.argument("locationPoint2"));
                RoutePath path = helper.makeRoute(point1, point2);
                result.success(Convert.routePathToObject(path));
                break;
            default:
                result.notImplemented();
                break;
        }
    }


    @Override
    public void onDetachedFromEngine(@NonNull FlutterPluginBinding binding) {

    }

    @Override
    public void onAttachedToActivity(@NonNull ActivityPluginBinding binding) {
        this.context = binding.getActivity();
        channel.setMethodCallHandler(this);
    }

    @Override
    public void onDetachedFromActivityForConfigChanges() {
    }

    @Override
    public void onReattachedToActivityForConfigChanges(@NonNull ActivityPluginBinding binding) {
        this.context = binding.getActivity();
        channel.setMethodCallHandler(this);
    }

    @Override
    public void onDetachedFromActivity() {
    }
}
