//
//  NavigineHelper.swift
//  navigine_sdk
//
//  Created by Aawaz on 25/09/2021.
//
import Navigine
import Foundation
import Flutter


class NavigineHelper: NSObject, NavigineCoreDelegate, NavigineCoreLocationDelegate, NavigineCoreBluetoothDelegate {
    
    func navigineCore(_ navigineCore: NavigineCore, didUpdateBluetoothState status: CBManagerState) {
        
    }
    
    
    var navigineCore: NavigineCore?
    var methodChannel: FlutterMethodChannel?
    
    func setMethodChannel(methodChannel channel: FlutterMethodChannel?) {
        self.methodChannel = channel
    }
    
    func initilizeSDK(_ userHash: String, serverURL: String?) {
        if(serverURL == nil){
            navigineCore = NavigineCore(userHash: userHash)
        } else {
            navigineCore = NavigineCore(userHash: userHash, server: serverURL!)
        }
        navigineCore?.locationDelegate =  self
        navigineCore?.bluetoothDelegate = self
    }
    
    func setDelegate(deviceInfoDelegate: NavigineCoreNavigationDelegate) {
        navigineCore?.navigationDelegate = deviceInfoDelegate
    }
    
    func attachNotificationListener(){
        navigineCore?.delegate = self
    }
    
    func removeNotificationListener(){
        navigineCore?.delegate = nil
    }
    
    func loadLocation(_ locationId: NSNumber?) {
        let locationIdInteger = locationId?.intValue ?? 0
        
        navigineCore?.downloadLocation(byId: locationIdInteger, forceReload: true, processBlock: {(_ loadProcess: Int) in let val = NSNumber(value: loadProcess)
            self.methodChannel?.invokeMethod("location#onUpdate", arguments: val)
        }, successBlock: {(_ userInfo:[AnyHashable : Any]?) in
            self.methodChannel?.invokeMethod("location#onFinished", arguments: nil)
        }, fail: { (_ error: Error?) in
            print(error)
        })
    }
    
    func startNavigation() {
        navigineCore?.startNavigine()
    }
    
    func stopNavigation() {
        navigineCore?.stopNavigine()
    }
    
    func getLocation() -> Any? {
        return Convert.locationInfo(toObject: navigineCore?.location)
    }
    
    func getSublocationImage(subLocationId: Int) -> Any? {
        let subLocation = navigineCore?.location.subLocation(withId: subLocationId)
        return subLocation?.sublocationImage.imageData
    }
    
    func getDeviceInfo() -> Any? {
        return Convert.deviceInfo(toObject: navigineCore?.deviceInfo)
    }
    
    func addTarget(locationPoint: NCLocationPoint) -> Any{
        return navigineCore?.addTarget(locationPoint)
    }
    
    func cancelTarget() -> Any {
        return navigineCore?.cancelTarget()
    }
    
    func didRangePush(withTitle title: String, content: String, image: String, id: Int) {
        var data: Dictionary<AnyHashable, AnyHashable> = [:]
        
        data["id"] = id;
        data["title"] = title;
        data["content"] = content;
        data["imageUrl"] = image;
        
        SwiftNavigineSdkPlugin.channel?.invokeMethod("marketing#onNotification", arguments: data)
    }
}
