part of navigine_sdk;

class Location {
  int id = 0;
  int version = 0;
  String name = "";
  String? description;
  List<SubLocation> subLocations = [];
  List<Category> categories = [];
  List<RoutePath> paths = [];
  String? mapFile;
  String? updateFile;
  String? actionsFile;

  SubLocation? getSubLocationById(int id) {
    return subLocations.firstWhereOrNull((sl) => sl.id == id);
  }

  NavigationThread? _thread;

  Future<NavigationThread> startNavigation({
    NavigationMode mode = NavigationMode.MODE_NORMAL,
  }) async {
    await _channel.invokeMethod("startNavigation", {
      "mode": _getModeCode(mode),
    });
    return _thread = NavigationThread._();
  }

  _handleMethodCall(MethodCall call) {
    switch (call.method) {
      case "marketing#onNotification":
        _notificationController
            ?.add(NavigineNotification.fromMap(call.arguments));
        break;
      case "deviceInfo#onUpdate":
        _thread?._handleNativeCall(call);
        break;
      default:
    }
  }

  static _getModeCode(NavigationMode mode) {
    switch (mode) {
      case NavigationMode.MODE_IDLE:
        return 1;
      case NavigationMode.MODE_SCAN:
        return 2;
      case NavigationMode.MODE_NORMAL:
        return 3;
      case NavigationMode.MODE_ECONOMIC:
        return 4;
      default:
    }
  }

  StreamController<NavigineNotification>? _notificationController;

  Stream<NavigineNotification> notificationStream() {
    _notificationController = StreamController();
    _channel.invokeMethod('attachNotificationListener');
    _notificationController!.onCancel = () async {
      await _channel.invokeMethod('removeNotificationListener');
      _notificationController?.close();
      _notificationController = null;
    };
    return _notificationController!.stream;
  }

  static Location fromMap(var data) {
    Location l = Location();
    l.id = data["id"];
    l.version = data["version"];
    l.name = data["name"];
    l.description = data["description"];
    l.mapFile = data["mapFile"];
    l.updateFile = data["updateFile"];
    l.actionsFile = data["actionsFile"];
    l.subLocations = data["subLocations"]
        .map<SubLocation>((sl) => SubLocation.fromMap(sl))
        .toList();

    l.categories = (data["categories"] ?? [])
        .map<Category>((sl) => Category.fromMap(sl))
        .toList();

    return l;
  }

  static Location fromAPIMap(var data) {
    return Location()
      ..name = data["title"]
      ..description = data["description"]
      ..id = data["id"];
  }
}
